using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using TMPro;
using UnityEngine.UI;
using System.IO.Ports;

public class SensorText : MonoBehaviour
{
    // Soil Moisture
    public TextMeshProUGUI sensorText;
    public Transform rainOrigin;
    public GameObject rainObject;
    private List<ParticleSystem> rains = new List<ParticleSystem>();
    public float moisture;
    public GameObject greenScene;
    public GameObject desertScene;
    public Image bar;

    // Water Level
    public bool tankFull = false;
    public TextMeshProUGUI waterLevelText;
    public GameObject checkbox;
    public GameObject xBox;
    public GameObject forestry;
    public GameObject desertry;

    // Pump Status
    public bool pumpIsOn;
    public TextMeshProUGUI pumpStatusText;
    public TextMeshProUGUI pumpButtonText;
    public ParticleSystem water;

    //General
    public bool inDebug = true;
    public SelectionMenu selMen;
    //SerialPort stream = new SerialPort("COM4", 9600);

    private void Start()
    {
        //stream.Close();
        //stream.Open();
        StartCoroutine(ReadSerial());
    }

    void Update()
    {
        UpdateSensorText();
    }

    IEnumerator ReadSerial()
    {
        yield return new WaitForSeconds(1);

        //Data RecievalSerialPort mySerialPort = new SerialPort("COM1");
        Process();
    }

    public void Process()
    {
        if (inDebug)
        {
            Update_MoistureVisual(moisture);
            Update_WaterLevel(tankFull);
            Update_PumpStatus(pumpIsOn);
            return;
        }

        SerialPort mySerialPort = new SerialPort(selMen.portSelected);
        mySerialPort.BaudRate = 9600;
        mySerialPort.Parity = Parity.None;
        mySerialPort.StopBits = StopBits.One;
        mySerialPort.DataBits = 8;
        mySerialPort.Handshake = Handshake.None;
        mySerialPort.Open();
        string value = mySerialPort.ReadLine();
        mySerialPort.Close();

        string[] recievedData = value.Split('&');
        if (recievedData.Length <= 1)
        {
            StartCoroutine(ReadSerial());
            return;
        }
        string[] subData = recievedData[1].Split('@');
        print(recievedData[1]);

        //Soil Moisture
        float soilMoisture = float.Parse(subData[0]);

        soilMoisture = Mathf.Abs(100 - (((soilMoisture - 230) / 223) * 100));
        soilMoisture = ((Mathf.RoundToInt(soilMoisture * 100)) / 100);

        if (soilMoisture > 100)
        {
            soilMoisture = 100;
        }
        else if (soilMoisture < 0)
        {
            soilMoisture = 0;
        }

        if (soilMoisture < 0)
        {
            soilMoisture = 0;
        }

        if (soilMoisture < 10)
        {
            TurnOnPump();
        }

        Update_MoistureVisual(soilMoisture);
        sensorText.text = "Soil Moisture: " + soilMoisture.ToString() + "%";

        //Water Level
        float waterLevel = float.Parse(subData[1]);

        if (waterLevel < 100)
        {
            Update_WaterLevel(false);
        }
        else
        {
            Update_WaterLevel(true);
        }

        //Pump Toggle
        int pumpEnable = int.Parse(subData[2]);
        bool pumpOn = false;

        if (pumpEnable == 1)
        {
            pumpOn = true;
        }
        else
        {
            pumpOn = false;
        }

        pumpIsOn = pumpOn;
        Update_PumpStatus(pumpIsOn);
        StartCoroutine(ReadSerial());
    }

    public void UpdateSensorText()
    {
        if (inDebug)
        {
            Update_MoistureVisual(moisture);
            Update_WaterLevel(tankFull);
            Update_PumpStatus(pumpIsOn);
            return;
        }
        return;

        
    }

    public void TurnOnPump()
    {
        SerialPort mySerialPort = new SerialPort("COM4");
        mySerialPort.BaudRate = 9600;
        mySerialPort.Parity = Parity.None;
        mySerialPort.StopBits = StopBits.One;
        mySerialPort.DataBits = 8;
        mySerialPort.Handshake = Handshake.None;
        mySerialPort.Open();
        mySerialPort.Write("1");
        mySerialPort.Close();
    }

    public void Update_PumpStatus(bool on)
    {
        if (on)
        {
            water.Play();
            pumpButtonText.text = "Pumping...";
            pumpStatusText.text = "Pump Status: PUMPING";
        }
        else
        {
            water.Stop();
            pumpButtonText.text = "Trigger Pump";
            pumpStatusText.text = "Pump Status: IDLE";
        }
    }

    public void Update_WaterLevel(bool full)
    {
        if (full)
        {
            xBox.SetActive(false);
            checkbox.SetActive(true);
            forestry.SetActive(true);
            desertry.SetActive(false);
            waterLevelText.text = "Tank Status: FULL";
        }
        else
        {
            xBox.SetActive(true);
            checkbox.SetActive(false);
            forestry.SetActive(false);
            desertry.SetActive(true);
            waterLevelText.text = "Tank Status: NEEDS WATER";
        }
    }

    public void Update_MoistureVisual(float reading)
    {
        if (reading > 100)
        {
            reading = 100;
        }
        else if (reading < 0)
        {
            reading = 0;
        }

        bar.fillAmount = reading / 100;
        sensorText.text = "Soil Moisture: " + reading.ToString() + "%";

        if (reading <= 20)
        {
            //Have 0 rains

            desertScene.SetActive(true);
            greenScene.SetActive(false);

            if (rains.Count > 0)
            {
                List<ParticleSystem> toDestroy = new List<ParticleSystem>();

                foreach (ParticleSystem rain in rains)
                {
                    toDestroy.Add(rain);
                }

                for (int i = 0; i < toDestroy.Count; i++)
                {
                    toDestroy[i].Stop();
                    rains.Remove(toDestroy[i]);
                }
            }
        }
        else if (reading <= 40 && reading > 20)
        {
            //Have 1 rain

            desertScene.SetActive(false);
            greenScene.SetActive(true);

            List<ParticleSystem> toDestroy = new List<ParticleSystem>();

            foreach (ParticleSystem rain in rains)
            {
                if (rain != rains[0])
                {
                    toDestroy.Add(rain);
                }
            }

            for (int i = 0; i < toDestroy.Count; i++)
            {
                toDestroy[i].Stop();
                rains.Remove(toDestroy[i]);
            }

            if (rains.Count == 0)
            {
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
            }
        }
        else if (reading <= 60 && reading > 40)
        {
            //Have 2 rains

            desertScene.SetActive(false);
            greenScene.SetActive(true);

            List<ParticleSystem> toDestroy = new List<ParticleSystem>();

            foreach (ParticleSystem rain in rains)
            {
                if (rain != rains[0] && rain != rains[1])
                {
                    toDestroy.Add(rain);
                }
            }

            for (int i = 0; i < toDestroy.Count; i++)
            {
                toDestroy[i].Stop();
                rains.Remove(toDestroy[i]);
            }

            if (rains.Count == 0)
            {
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
            }
            else if (rains.Count == 1)
            {
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
            }
        }
        else if (reading <= 80 && reading > 60)
        {
            //Have 3 rains

            desertScene.SetActive(false);
            greenScene.SetActive(true);

            List<ParticleSystem> toDestroy = new List<ParticleSystem>();

            foreach (ParticleSystem rain in rains)
            {
                if (rain != rains[0] && rain != rains[1] && rain != rains[2])
                {
                    toDestroy.Add(rain);
                }
            }

            for (int i = 0; i < toDestroy.Count; i++)
            {
                toDestroy[i].Stop();
                rains.Remove(toDestroy[i]);
            }

            if (rains.Count == 0)
            {
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
            }
            else if (rains.Count == 1)
            {
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
            }
            else if (rains.Count == 2)
            {
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
            }
        }
        else if (reading <= 100 && reading > 80)
        {
            //Have 4 rains

            desertScene.SetActive(false);
            greenScene.SetActive(true);

            List<ParticleSystem> toDestroy = new List<ParticleSystem>();

            foreach (ParticleSystem rain in rains)
            {
                if (rain != rains[0] && rain != rains[1] && rain != rains[2] && rain != rains[3])
                {
                    toDestroy.Add(rain);
                }
            }

            for (int i = 0; i < toDestroy.Count; i++)
            {
                toDestroy[i].Stop();
                rains.Remove(toDestroy[i]);
            }

            if (rains.Count == 0)
            {
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
            }
            else if (rains.Count == 1)
            {
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
            }
            else if (rains.Count == 2)
            {
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
            }
            else if (rains.Count == 3)
            {
                rains.Add(Instantiate(rainObject, rainOrigin.transform.position, rainOrigin.transform.rotation, rainOrigin.transform).GetComponent<ParticleSystem>());
            }
        }
    }

}
